/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

 *****************************************************************************/

#include <stdio.h>
#include "depend.h"
#include "param.h"
#include "promisc.h"
#include "analyze.h"
#include "gddconfig.h"
#include "error.h"
#include "gui_win32_console.h"
#include "gui_win32_msghdlr_main.h"
#include "gdd_printf.h"
#include "resource.h"
#include <commctrl.h>

#define MYWM_NTFYICON   WM_APP+100

/*============================================================================
 Callback procedure of main Dialog Box
 [INPUT]
  hWnd      : Window handle of main window
  msg       : Message
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
  none
  ============================================================================
*/

LRESULT CALLBACK WndProc(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
    BOOL TrayMessage(HWND hDlg,DWORD dwMessage);

    static GDDCONFIG    *gddc,bak_gddc,org_gddc;
    static int          switems[]={IDC_SRCIP,IDC_DSTIP,IDC_INTERFACE,IDC_PORT,
                                   IDC_HOLDTMPLOG,IDC_PKTDUMP,IDC_SECUREMODE,-1};
    int                 i;
    static int          sizeflag;
    NM_LISTVIEW         *pNM;
    HANDLE              hFind;
    WIN32_FIND_DATA     fd;
    char                dirbuf[MAX_PATH],buf[MAX_PATH];

    switch(Msg){

    case WM_INITDIALOG:
        InitCommonControls();
        gddc=(GDDCONFIG *)lParam;
        gddc->egui.hWnd=hWnd;
        if (Win32MsgHdlr_InitDialog(gddc,wParam,lParam)!=MH_SUCCESS)
            MessageBox(hWnd,"Initialization error","ERROR",MB_OK);
        if (Win32MsgSubfunc_SetParamsToMainDlg(gddc)==0)
            errorlog(ERRLOG_WARNING,"Network interface is not installed\n");
        gddc->egui.console_mode=CNSLMD_LIST;
        if (gddc->hold)
            Win32MsgHdlr_CmdHoldTmpLog(gddc,1,0);
        TrayMessage(hWnd,NIM_ADD);
        Win32MsgSubfunc_LED(gddc,0,LED_INIT,0);
        sizeflag=0;
        CreateDirectory(WORKDIR,NULL);
		memcpy(&org_gddc,gddc,sizeof(org_gddc));
        break;
    case WM_ASYNC_SELECT:
        Win32MsgSubfunc_LED(gddc,LED2,LED_ON,50);
        if (Win32MsgHdlr_AsyncSelect(gddc,wParam,lParam)==MH_ANALYSISERR){
            gdd_printf("# Packet handling error\n");
            Win32MsgHdlr_CmdStop(gddc,wParam,lParam);
        }
        break;
    case WM_DESTROY:
        Win32MsgSubfunc_LED(gddc,0,LED_TERM,0);
        TrayMessage(hWnd, NIM_DELETE);
        _snprintf(dirbuf,MAX_PATH-1,"%s\\*.*",WORKDIR);
        dirbuf[MAX_PATH-1]='\0';
        hFind = FindFirstFile(dirbuf, &fd);
        if(hFind != INVALID_HANDLE_VALUE){
            do {
                if (strcmp(fd.cFileName,".") && strcmp(fd.cFileName,"..")){
                    _snprintf(buf,MAX_PATH-1,"%s\\%s",WORKDIR,fd.cFileName);
                    DeleteFile(buf);
                }
            }while(FindNextFile(hFind, &fd));
            FindClose(hFind);
        }
        RemoveDirectory(WORKDIR);
        WSACleanup();
        break;
    case WM_TIMER:
        Win32MsgHdlr_Timer(gddc,wParam,lParam);
        break;
    case WM_CTLCOLORSTATIC:
        for (i=0;i<=3;i++){
            if ((HWND)lParam==GetDlgItem(hWnd,switems[i]))
                return((LRESULT)GetStockObject(WHITE_BRUSH));
        }
        if ((HWND)lParam==GetDlgItem(hWnd,IDC_CONSOLE))
            return((LRESULT)GetStockObject(WHITE_BRUSH));
        else
            break;
    case WM_COMMAND:
        switch(GET_WM_COMMAND_ID(wParam, lParam)){
            case IDCANCEL:
				if (Win32MsgSubfunc_GetParamsFromMainDlg(gddc,0)!=0){
					i=MessageBox(hWnd,"Can not save due to parameter error(s). Do you really quit?",
                        "Warning",MB_YESNO);
					if (i==IDYES) EndDialog(hWnd,0);
					else break;
				}
				if (memcmp(&org_gddc,gddc,sizeof(org_gddc))){
					i=MessageBox(hWnd,"Save current settings?",
                        "Save Settings",MB_YESNO);
					if (i==IDYES)
						Win32MsgSubfunc_SaveCFG(gddc);
				}
                EndDialog(hWnd,0);
                break;
            case IDC_GO:
                if (Win32MsgSubfunc_GetParamsFromMainDlg(gddc,1)!=0
                    ||Win32MsgHdlr_CmdGo(gddc,wParam,lParam)!=MH_SUCCESS){
                    gdd_printf("# Can not start capture\n");
                    break;
                }else
                    gdd_printf("# Capture started\n");
                EnableWindow(GetDlgItem(hWnd,IDC_GO),FALSE);
                EnableWindow(GetDlgItem(hWnd,IDC_CFG),FALSE);
                EnableWindow(GetDlgItem(hWnd,IDC_STOP),TRUE);
                for (i=0;;i++){
                    if (switems[i]==-1) break;
                    EnableWindow(GetDlgItem(hWnd,switems[i]),FALSE);
                }
                Win32MsgSubfunc_LED(gddc,LED1,LED_ON,LEDTIMER_INF);
                break;
            case IDC_STOP:
                if (enum_connection(gddc)!=0){
                    i=MessageBox(hWnd,"Some connections are still active. Clean up ?",
                        "Warning",MB_YESNO);
                    if (i==IDYES)
                        fourceterm_connection(gddc);
                }
                if (Win32MsgHdlr_CmdStop(gddc,wParam,lParam)!=MH_SUCCESS){
                    gdd_printf("# Abnormal termination\n");
                    break;
                }else
                    gdd_printf("# Capture terminated\n");
                EnableWindow(GetDlgItem(hWnd,IDC_GO),TRUE);
                EnableWindow(GetDlgItem(hWnd,IDC_CFG),TRUE);
                EnableWindow(GetDlgItem(hWnd,IDC_STOP),FALSE);
                for (i=0;;i++){
                    if (switems[i]==-1) break;
                    EnableWindow(GetDlgItem(hWnd,switems[i]),TRUE);
                }
                Win32MsgSubfunc_LED(gddc,LED1,LED_OFF,LEDTIMER_INF);
                break;
            case IDC_CFG:
                if (Win32MsgHdlr_CmdCfg(gddc,wParam,lParam)!=MH_SUCCESS)
                    gdd_printf("# Configuration error\n");
                break;
            case IDC_HOLDTMPLOG:
                Win32MsgHdlr_CmdHoldTmpLog(gddc,(WPARAM)IsDlgButtonChecked(hWnd,IDC_HOLDTMPLOG),0);
                break;
        }
        break;
    case MYWM_NTFYICON:
        switch (lParam){
            case WM_LBUTTONDOWN:
                if (sizeflag==1){
                    ShowWindow(hWnd, SW_SHOWNORMAL);
                    SetForegroundWindow(hWnd);
                    sizeflag=0;
                }else{
                    ShowWindow(hWnd, SW_MINIMIZE);
                    ShowWindow(hWnd, SW_HIDE);
                    sizeflag=1;
                }
                break;

            default:
                break;
        }
        break;
    case WM_NOTIFY:
        pNM=(NM_LISTVIEW*)lParam;
        if (pNM->hdr.code==NM_DBLCLK && pNM->iItem>=0){
            Win32MsgSubfunc_SelectList(gddc,pNM->iItem);
            break;
        }
        break;
    default:
        return FALSE;
    }
    return TRUE;
}

/*============================================================================
  Set this app to tasktray

 [Input]
    hDlg        : Window handle
    dwMessage   : Message to Shell_NofityIcon
 [Output]
    TRUE = Success
    FALSE = Fail
 ============================================================================
*/
BOOL TrayMessage(HWND hDlg,DWORD dwMessage)
{    
    BOOL            res;
    NOTIFYICONDATA  tnd;
    char            lpszTip[200];
    HICON           hIcon;
    HINSTANCE       hInst=GetModuleHandle(0);

    hIcon=(HICON)LoadImage(hInst,MAKEINTRESOURCE(IDI_ICON),IMAGE_ICON,16,16,0);
    sprintf(lpszTip,"%s %s",CLASSNAME,GDD_VERSION);
    tnd.cbSize              = sizeof(NOTIFYICONDATA);
    tnd.hWnd                = hDlg;
    tnd.uID                 = IDCANCEL;
    tnd.uFlags              = NIF_MESSAGE|NIF_ICON|NIF_TIP;
    tnd.uCallbackMessage    = MYWM_NTFYICON;
    tnd.hIcon               = hIcon;
    if (lpszTip) 
        lstrcpyn(tnd.szTip, lpszTip, sizeof(tnd.szTip)); 
    else 
        tnd.szTip[0] = '\0'; 

    res=Shell_NotifyIcon(dwMessage, &tnd);
    if (hIcon)
        DestroyIcon(hIcon);
    return res;
}

/*============================================================================
 Callback procedure of Dummy Window window
 [INPUT]
  hWnd      : Window handle of main window
  msg       : Message
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
  none
  ============================================================================
*/
LRESULT CALLBACK WndProc_dummywindow(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam)
{
    static GDDCONFIG        *gddc;

    switch(Msg){

    case WM_CREATE:
        gddc=(GDDCONFIG *)(((LPCREATESTRUCT) lParam)->lpCreateParams); 
        gddc->egui.hWnd=hWnd;
        if (Win32MsgHdlr_CmdGo(gddc,wParam,lParam)!=MH_SUCCESS)
            PostQuitMessage(0);
        return 0;
    case WM_ASYNC_SELECT:
        if (Win32MsgHdlr_AsyncSelect(gddc,wParam,lParam)==MH_ANALYSISERR)
             PostQuitMessage(0);
        break;
    case WM_DESTROY:
        WSACleanup();
        PostQuitMessage(0);
        return 0;
    }
    return DefWindowProc(hWnd, Msg, wParam, lParam);
}

