/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 *****************************************************************************/

#include <ctype.h>
#include <string.h>
#include "depend.h"
#include "promisc.h"

/*============================================================================
 Set network interface to promiscus mode.
 [INPUT]
  nit_dev       : NIT device name
  nic_name      : Name of network interface  
  chunksize     : Chunk size
 [RETURN]
  ERR_PROMISC_OPEN              : Can not open device 
  ERR_PROMISC_READ              : Can not read status 
  ERR_PROMISC_WRITE             : Can not write status 
  ERR_PROMISC_BIND              : Can not bind
  ERR_PROMISC_SETSOCKOPT        : setsockopt failure
  ERR_PROMISC_TOOLONG_NITDEV    : Too long nit_dev
  ERR_PROMISC_TOOLONG_NICNAME   : Too long nic_name
  otherwise                     : device handle
  ============================================================================
*/
SOCKET setnic_promisc(char *nit_dev,char *nic_name,int *chunksize)
{
	int             sock;
	struct ifreq    f;
	struct strioctl si;
	struct timeval  timeout;
	unsigned int    chunksize;
	unsigned long   if_flags;

	/* Open device */
	if ((sock=open(nit_dev,O_RDONLY))<0)
		return(ERR_PROMISC_OPEN);

	/* Arrange to get discrete messages from the stream */
	if (ioctl(sock,I_SRDOPT,(char *)RMSGD)<0)
		return(ERR_PROMISC_WRITE);

	/* Push and configure the buffering module */
	if (ioctl(sock,I_PUSH,"nbuf")<0)
		return(ERR_PROMISC_WRITE);

	timeout.tv_sec  = 1;
	timeout.tv_usec = 0;
	si.ic_timout    = INFTIM;
	si.ic_cmd       = NIOCSTIME;
	si.ic_len       = sizeof(timeout);
	si.ic_dp        = (char *)&timeout;
	if (ioctl(sock,I_STR,(char *)&si)<0)
		return(ERR_PROMISC_WRITE);

	chunksize       = CHUNKSIZE;
	si.ic_cmd       = NIOCSCHUNK;
	si.ic_len       = sizeof(chunksize);
	si.ic_dp        = (char *)&chunksize;
	if (ioctl(sock,I_STR,(char *)&si)<0)
		return(ERR_PROMISC_WRITE);

	/* Bind */
	strncpy(f.ifr_name, nic_name, sizeof(f.ifr_name));
	f.ifr_name[sizeof(f.ifr_name) - 1] = '\0';
	si.ic_cmd       = NIOCBIND;
	si.ic_len       = sizeof(f);
	si.ic_dp        = (char *)&f;
	if (ioctl(sock,I_STR,(char *)&si)<0)
		return(ERR_PROMISC_BIND);

	/* Set into Promiscus mode */
	if_flags        = NI_PROMISC;
	si.ic_cmd       = NIOCSFLAGS;
	si.ic_len       = sizeof(if_flags);
	si.ic_dp        = (char *)&if_flags;
	if (ioctl(sock,I_STR,(char *)&si)<0)
		return(ERR_PROMISC_SETSOCKOPT);

	/* Flush r/w queue */
	if (ioctl(sock, I_FLUSH, (char *)FLUSHR)<0)
		return(ERR_PROMISC_WRITE);

	return(sock);
}
